package HostPublisher;
import java.awt.*;
import java.util.Vector;
import java.util.Enumeration;
import java.lang.Thread;
import javax.servlet.http.*;
import com.ibm.HostPublisher.IntegrationObject.HPubCommon;


/**
 * <P>ParallelHPIO is a JavaBean which allows you to invoke two
 * or more Host Publisher Integration Objects in parallel.
 * This is useful if multiple Integration Objects can run
 * independent of each other (meaning that one object does
 * not depend on the output or state of another, such as
 * through Integration Object chaining), allowing the objects
 * to be invoked at the same time.
 * 
 * <P>You simply give an instance of ParallelHPIO pointers to each
 * Integration Object you want to run, then tell it to run
 * them. It will return once all Integration Objects have
 * completed.
 * 
 * <P>This class only works with Host Publisher generated 
 * Integration Objects, but can easily be modified to invoke
 * any kind of object in parallel.
 * 
 * <h4><b>Example</b></h4>
 * <P>The most common implementation of ParallelHPIO JavaBean will
 * be on a JSP page. To use the bean, follow these steps.
 * <ol>
 * <li>Download the ParallelHPIO.jar file and add it to 
 * your CLASSPATH for the Host Publisher Studio's environment.
 * For example: <pre>CLASSPATH=%CLASSPATH%;c:\hostpub\studio\ParallelHPIO.jar</pre>
 * <li>Within your Host Publisher Studio application, import an
 * Integration Object from the File menu. Locate this JAR file and
 * click OK.
 * <li>Host Publisher Studio will prompt you to select the class file out of the JAR to
 * import. Select HostPublisher.ParallelHPIO.class.
 * <li>You'll be asked to identify this bean's inputs, outputs,
 * and execution method. You won't actually make use of any of these,
 * so you can click OK without specifying anything.
 * <li>On the page where you want to add the ParallelHPIO bean,
 * place the cursor in the location where the bean is to be
 * located, click the right mouse button and select Insert
 * Integration Object. 
 * <li>Select ParallelHPIO as the object to insert and keep
 * clicking Next until you can click Finish.
 * <li>All of the objects that should be invoked in parallel, as
 * well as the ParallelHPIO bean should be on the page now.
 * You must now remove the calls to doHPTransaction() for each
 * individual Integration Object. These calls will be made by ParallelHPIO.
 * <li>Add the appropriate calls to the ParallelHPIO object
 * to properly run all of the Integration Objects in parallel:
 * <pre>
 * &lt;BEAN NAME="HAParallelTest" TYPE="IntegrationObject.HAParallelTest" INTROSPECT="yes" CREATE="yes" SCOPE="request"&gt;&lt;/BEAN&gt;
 * &lt;BEAN NAME="HAParallelTest2" TYPE="IntegrationObject.HAParallelTest" INTROSPECT="yes" CREATE="yes" SCOPE="request"&gt;&lt;/BEAN&gt;
 * &lt;BEAN NAME="HAParallelTest3" TYPE="IntegrationObject.HAParallelTest" INTROSPECT="yes" CREATE="yes" SCOPE="request"&gt;&lt;/BEAN&gt;
 * &lt;BEAN NAME="ParallelHPIO" TYPE="HostPublisher.ParallelHPIO" INTROSPECT="yes" CREATE="yes" SCOPE="request"&gt;&lt;/BEAN&gt;
 * &lt;% 
 *    ParallelHPIO.setSessionInfo(request, response);
 *    ParallelHPIO.addIO(HAParallelTest);
 *    ParallelHPIO.addIO(HAParallelTest2);
 *    ParallelHPIO.addIO(HAParallelTest3);
 *    ParallelHPIO.runIOs(); %&gt;
 * </pre>
 * <li>Be sure to set all Integration Object inputs before the runIOs() method is invoked.
 * <li>When you transfer the application to the server, you
 * might get a complaint that ParallelHPIO.macro could not
 * be found. You can ignore this error.
 * <li>Deploy the application and try it out. For Host
 * Access Integration Objects, you can switch on the Terminal
 * trace option under Host Connection Tracing to see all of the
 * terminal windows opening in parallel.
 * </ol>
 * <p>Note that the Integration Objects <b><i>must</b></i> be
 * able to be run in parallel. Make sure that if pooling is
 * turned on that the pool is configured with a high enough
 * maximum for the number of connections to support parallelism.
 * Also be careful of chained Integration Objects running
 * in parallel that might be dependent on another's state
 * before starting.
 * <P><B>NOTE: This object and source is provided AS IS as
 * a productivity aid to customers. While feedback is 
 * welcome and encouraged, IBM is in no way obligated to 
 * support or service this feature. The source is provided
 * so that you may make modifications as appropriate.</B> 
 * Feedback may be provided through the Internet newsgroup 
 * ibm.software.hostpubv2 on news.software.ibm.com.
 */
public class ParallelHPIO extends java.lang.Object
{

    /**
     * Maintains a list of Integration Objects which are to
     * be invoked by this object.
     */
    Vector ioList = new Vector();

    /**
     * An instance of HttpServletRequest used to invoke all
     * Integration Objects' doHPTransaction() method.
     */
    HttpServletRequest request;

    /**
     * An instance of HttpServletResponse used to invoke all
     * Integration Objects' doHPTransaction() method.
     */
    HttpServletResponse response;
	/**
	 * Constructor.
	 */
    
	public ParallelHPIO()
	{
	    request = null;
	    response = null;
	}
	/**
	 * <P>Provides ParallelHPIO with the HttpServletResponse and
	 * HttpServletRequest objects to use when invoking the
	 * Integration Objects. Integration Objects are invoked by
	 * calling their doHPTransaction() method, which requires
	 * these objects as input.
	 * 
	 * <P>The same objects are used for every Integration Object
	 * invocation.
	 * 
	 * <P>These objects are created and maintained by WebSphere.
	 * 
	 * @param req The HttpServletRequest object.
	 * @param res The HttpServletResponse object.
	 */
	
	public void setSessionInfo(HttpServletRequest req, HttpServletResponse res)
	{
	    request = req;
	    response = res;
	}

    /**
     * Adds an Integration Object instance to a vector of objects.
     * 
     * @param io An instance of an Integration Object to add to the list.
     */
    public void addIO(HPubCommon io)
    {
        ioList.addElement(io);
    }
    
    /**
     * Invokes Integration Objects as they appear in the list
     * (populated by calls to addIOs()) by generating a thread
     * for each Integration Object which invokes that object's
     * doHPTransaction() method. The threads are all started and
     * stored in a list. The status of each thread in the list
     * is checked until all threads have stopped, then this
     * call returns.
     */
    public void runIOs()
    {
        Vector threads = new Vector();
        
        // kick off an individual thread for each IO's doHPTransaction()
        for (Enumeration e = ioList.elements(); e.hasMoreElements(); )
        {
            IOThread thread = new IOThread((HPubCommon)e.nextElement());
            thread.start();
            threads.addElement(thread);
        }
        
        // now check the threads until all are not running
        boolean running = true;
        while (running)
        {
            try {
                Thread.sleep(100);
            } catch (Exception e) {}
            running = false;
            for (Enumeration e1 = threads.elements(); e1.hasMoreElements() && !running; )
            {
                running = ((IOThread)e1.nextElement()).isAlive();
            }
        }
        
        // all threads are done, so return
    }
    
    class IOThread extends Thread
    {
        HPubCommon io;
        
        IOThread()
        {
            super("IOThread");
            setPriority(MAX_PRIORITY);
            io = null;
        }
        
        IOThread(HPubCommon ioToUse)
        {
            super("IOThread");
            setPriority(MAX_PRIORITY);
            io = ioToUse;
        }
        
        public void run()
        {
            if (io != null)
                io.doHPTransaction(request, response);
        }
    }
}
